""" Manages student repositories when using Github sumission system """
import shutil
import subprocess
import sys
import re

from .utils import print_color, TermColors


def clone_repo(git_path, tag, student_repo_path):
    """Clone the student repository"""

    if student_repo_path.is_dir() and list(student_repo_path.iterdir()):
        print_color(
            TermColors.BLUE,
            "Student repo",
            student_repo_path.name,
            "already cloned. Re-fetching tag",
        )

        # Fetch
        cmd = ["git", "fetch", "--tags", "-f"]
        try:
            subprocess.run(cmd, cwd=student_repo_path, check=True)
        except subprocess.CalledProcessError:
            print_color(TermColors.RED, "git fetch failed")
            return False

        # Checkout tag
        if tag not in ("master", "main"):
            tag = "tags/" + tag
        cmd = ["git", "checkout", tag, "-f"]
        try:
            subprocess.run(cmd, cwd=student_repo_path, check=True)
        except subprocess.CalledProcessError:
            print_color(TermColors.RED, "git checkout of tag failed")
            return False
        return True

    print_color(TermColors.BLUE, "Cloning repo, tag =", tag)
    cmd = [
        "git",
        "clone",
        "--branch",
        tag,
        git_path,
        str(student_repo_path.absolute()),
    ]
    try:
        subprocess.run(cmd, check=True)
    except KeyboardInterrupt:
        shutil.rmtree(str(student_repo_path))
        sys.exit(-1)
    except subprocess.CalledProcessError:
        print_color(TermColors.RED, "Clone failed")
        return False
    return True


def convert_github_url_format(url, to_https):
    """ " Convert a github URL to either HTTPS or SSH format

    If to_https is True, URLs will be converted to https://github.com/org/repo format:

    >>> convert_github_url_format("git@github.com:byu-ecen123-classroom/123-labs-username01.git", True)
    'https://github.com/byu-ecen123-classroom/123-labs-username01'

    If to_https is False then SSH format is assumed, and URLs will be converted to git@github.com:org/repo format:

    >>> convert_github_url_format("https://github.com/byu-ecen123-classroom/123-labs-username01", False)
    'git@github.com:byu-ecen123-classroom/123-labs-username01.git'

    It also works if the student provides the repo with ".git" extension

    >>> convert_github_url_format("https://github.com/byu-ecen123-classroom/123-labs-username01.git", True)
    'https://github.com/byu-ecen123-classroom/123-labs-username01'

    Any invalid format is returned with modification:

    >>> convert_github_url_format("invalid format", True)
    'invalid format'

    """
    org = repo = None

    match = re.search(r"git@github\.com:(.*?)/(.*?).git", url)
    if match:
        org = match.group(1)
        repo = match.group(2)
    match = re.search(r"github\.com/(.*?)/(.*)", url)
    if match:
        org = match.group(1)
        repo = match.group(2)

    # Remove .git
    if repo is not None and repo.endswith(".git"):
        repo = repo[:-4]

    if org is not None:
        if to_https:
            return "https://github.com/" + org + "/" + repo
        return "git@github.com:" + org + "/" + repo + ".git"
    return url


def print_date(student_repo_path):
    """Print the last commit date to the repo"""
    print("Last commit: ")
    cmd = ["git", "log", "-1", r"--format=%cd"]
    subprocess.run(cmd, cwd=str(student_repo_path), check=False)



#!/usr/bin/python3

import unittest
import pathlib
import sys
import time


ROOT_PATH = pathlib.Path(__file__).resolve().parent.parent
sys.path.append(str(ROOT_PATH))

from ygrader import Grader, CodeSource

TEST_PATH = ROOT_PATH / "test"
TEST_RESOURCES_PATH = TEST_PATH / "resources"


def run_on_lab(**kwargs):
    print("run_on_lab")
    time.sleep(1)


def run_on_milestone(**kwargs):
    print("run_on_milestone")
    time.sleep(1)


def test_me():
    grader = Grader(
        lab_name="lab1",
        grades_csv_path=TEST_RESOURCES_PATH / "grades1.csv",
        work_path=TEST_PATH / "temp",
    )
    grader.add_item_to_grade(
        "lab1", run_on_milestone, 10, help_msg="This is a long string\nWith lots of advice\nFor TAs"
    )
    grader.add_item_to_grade(
        "lab1m2",
        run_on_milestone,
        help_msg=[
            "msg2",
        ],
    )
    grader.set_submission_system_learning_suite(TEST_RESOURCES_PATH / "submissions.zip")
    grader.set_other_options(prep_fcn=run_on_lab)
    grader.run()


if __name__ == "__main__":
    test_me()



""" Separate script used to merge new changes to base repo into Githug classroom repos."""

import pathlib
import subprocess

import pandas

from . import utils, student_repos
from .utils import TermColors, error, print_color


class UpstreamMerger:
    """
    This class is used to automatically merge in new changes to your upstream repo into each student's repository.
    """

    def __init__(
        self, github_csv_path: pathlib.Path, github_csv_col_name: str, upstream_repo_url: str
    ):
        utils.check_file_exists(github_csv_path)

        self.github_csv_path = github_csv_path
        self.github_csv_col_name = github_csv_col_name
        self.upstream_repo_url = upstream_repo_url

    def run(self):
        """Run the merger process"""
        df = pandas.read_csv(self.github_csv_path)

        if self.github_csv_col_name not in df.columns:
            error(
                "Specified column name", self.github_csv_col_name, "does not exist in the CSV file"
            )
        if "Net ID" not in df.columns:
            error("CSV file does not contain column 'Net ID'")

        tmp_path = pathlib.Path.cwd() / "temp" / "upstream_merge"

        tmp_path.mkdir(exist_ok=True, parents=True)

        for _, row in df.iterrows():
            netid = row["Net ID"]
            print_color(TermColors.PURPLE, netid)

            student_tmp_path = tmp_path / netid
            student_repos.clone_repo(row[self.github_csv_col_name], "main", student_tmp_path)

            # Add remote
            subprocess.run(
                ["git", "remote", "add", "upstream", self.upstream_repo_url],
                cwd=student_tmp_path,
                check=False,
            )

            # Fetch remote
            subprocess.run(
                ["git", "fetch", "upstream"],
                cwd=student_tmp_path,
                check=True,
            )

            # Merge remote
            try:
                subprocess.run(
                    ["git", "merge", "upstream/main", "--no-edit"],
                    cwd=student_tmp_path,
                    check=True,
                )
            except subprocess.CalledProcessError:
                print_color(TermColors.RED, "merge failed.")
                break

            # Push
            subprocess.run(
                ["git", "push"],
                cwd=student_tmp_path,
                check=True,
            )
